@@ -1,6 +1,6 @@
 # pyenchant
 #
-# Copyright (C) 2004-2008, Ryan Kelly
+# Copyright (C) 2004-2011, Ryan Kelly
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -28,66 +28,73 @@
 # do so, delete this exception statement from your version.
 #
 """
-    enchant:  Access to the enchant spellchecking library
+enchant:  Access to the enchant spellchecking library
+=====================================================
 
-    This module provides several classes for performing spell checking
-    via the Enchant spellchecking library.  For more details on Enchant,
-    visit the project website:
+This module provides several classes for performing spell checking
+via the Enchant spellchecking library.  For more details on Enchant,
+visit the project website:
 
-        http://www.abisource.com/enchant/
+    http://www.abisource.com/enchant/
 
-    Spellchecking is performed using 'Dict' objects, which represent
-    a language dictionary.  Their use is best demonstrated by a quick
-    example:
+Spellchecking is performed using 'Dict' objects, which represent
+a language dictionary.  Their use is best demonstrated by a quick
+example::
 
-        >>> import enchant
-        >>> d = enchant.Dict("en_US")   # create dictionary for US English
-        >>> d.check("enchant")
-        True
-        >>> d.check("enchnt")
-        False
-        >>> d.suggest("enchnt")
-        ['enchant', 'enchants', 'enchanter', 'penchant', 'incant', 'enchain', 'enchanted']
+    >>> import enchant
+    >>> d = enchant.Dict("en_US")   # create dictionary for US English
+    >>> d.check("enchant")
+    True
+    >>> d.check("enchnt")
+    False
+    >>> d.suggest("enchnt")
+    ['enchant', 'enchants', 'enchanter', 'penchant', 'incant', 'enchain', 'enchanted']
 
-    Languages are identified by standard string tags such as "en" (English)
-    and "fr" (French).  Specific language dialects can be specified by
-    including an additional code - for example, "en_AU" refers to Australian
-    English.  The later form is preferred as it is more widely supported.
+Languages are identified by standard string tags such as "en" (English)
+and "fr" (French).  Specific language dialects can be specified by
+including an additional code - for example, "en_AU" refers to Australian
+English.  The later form is preferred as it is more widely supported.
 
-    To check whether a dictionary exists for a given language, the function
-    'dict_exists' is available.  Dictionaries may also be created using the
-    function 'request_dict'.
+To check whether a dictionary exists for a given language, the function
+'dict_exists' is available.  Dictionaries may also be created using the
+function 'request_dict'.
 
-    A finer degree of control over the dictionaries and how they are created
-    can be obtained using one or more 'Broker' objects.  These objects are
-    responsible for locating dictionaries for a specific language.
+A finer degree of control over the dictionaries and how they are created
+can be obtained using one or more 'Broker' objects.  These objects are
+responsible for locating dictionaries for a specific language.
     
-    In Python 2.x, unicode strings are supported transparently in the
-    standard manner - if a unicode string is given as an argument, the
-    result will be a unicode string. Note that Enchant works in UTF-8 
-    internally, so passing an ASCII string to a dictionary for a language
-    requiring Unicode may result in UTF-8 strings being returned.
+In Python 2.x, unicode strings are supported transparently in the
+standard manner - if a unicode string is given as an argument, the
+result will be a unicode string. Note that Enchant works in UTF-8 
+internally, so passing an ASCII string to a dictionary for a language
+requiring Unicode may result in UTF-8 strings being returned.
 
-    In Python 3.x unicode strings are expected throughout.  Bytestrings
-    should not be passed into any functions.
+In Python 3.x unicode strings are expected throughout.  Bytestrings
+should not be passed into any functions.
 
-    Errors that occur in this module are reported by raising subclasses
-    of 'Error'.
+Errors that occur in this module are reported by raising subclasses
+of 'Error'.
 
 """
-_DOC_ERRORS = ['enchnt','enchnt', 'fr']
+_DOC_ERRORS = ['enchnt','enchnt','incant','fr']
 
 # Make version info available
 __ver_major__ = 1
 __ver_minor__ = 6
-__ver_patch__ = 5
+__ver_patch__ = 6
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
                               __ver_patch__,__ver_sub__)
 
 import os
 
-from enchant import _enchant as _e
+try:
+    from enchant import _enchant as _e
+except ImportError:
+    if not os.environ.get("PYENCHANT_IGNORE_MISSING_LIB",False):
+        raise
+    _e = None
+
 from enchant.errors import *
 from enchant.utils import EnchantStr, get_default_language
 from enchant.pypwl import PyPWL
@@ -95,7 +102,7 @@ from enchant.pypwl import PyPWL
 #  Due to the unfortunate name collision between the enchant "tokenize" module
 #  and the stdlib "tokenize" module, certain values of sys.path can cause
 #  the former to override the latter and break the "warnings" module.
-#  This hacks around it by making a dumming "warnings" module.
+#  This hacks around it by making a dummy "warnings" module.
 try:
     import warnings
 except ImportError:
@@ -155,15 +162,23 @@ class _EnchantObject(object):
     def __init__(self):
         """_EnchantObject constructor."""
         self._this = None
+        #  To be importable when enchant C lib is missing, we need
+        #  to create a dummy default broker.
+        if _e is not None:
+            self._init_this()
         
     def _check_this(self,msg=None):
-         """Check that self._this is set to a pointer, rather than None."""
-         if msg is None:
-            msg = "%s unusable: the underlying C-library object has been freed."
-            msg = msg % (self.__class__.__name__,)
-         if self._this is None:
+        """Check that self._this is set to a pointer, rather than None."""
+        if self._this is None:
+            if msg is None:
+               msg = "%s unusable: the underlying C-library object has been freed."
+               msg = msg % (self.__class__.__name__,)
             raise Error(msg)
-             
+
+    def _init_this(self):
+        """Initialise the underlying C-library object pointer."""
+        raise NotImplementedError
+
     def _raise_error(self,default="Unspecified Error",eclass=Error):
          """Raise an exception based on available error messages.
 
@@ -176,6 +191,20 @@ class _EnchantObject(object):
          raise eclass(default)
     _raise_error._DOC_ERRORS = ["eclass"]
 
+    def __getstate__(self):
+        """Customize pickling of PyEnchant objects.
+
+        Since it's not safe for multiple objects to share the same C-library
+        object, we make sure it's unset when pickling.
+        """
+        state = self.__dict__.copy()
+        state["_this"] = None
+        return state
+
+    def __setstate__(self,state):
+        self.__dict__.update(state)
+        self._init_this()
+
 
 
 class Broker(_EnchantObject):
@@ -202,15 +231,23 @@ class Broker(_EnchantObject):
         arguments are required.
         """
         _EnchantObject.__init__(self)
+
+    def _init_this(self):
         self._this = _e.broker_init()
         if not self._this:
             raise Error("Could not initialise an enchant broker.")
+        self._live_dicts = {}
 
     def __del__(self):
         """Broker object destructor."""
         if _e is not None:
-          self._free()
-            
+            self._free()
+
+    def __getstate__(self):
+        state = super(Broker,self).__getstate__()
+        state.pop("_live_dicts")
+        return state
+
     def _raise_error(self,default="Unspecified Error",eclass=Error):
         """Overrides _EnchantObject._raise_error to check broker errors."""
         err = _e.broker_get_error(self._this)
@@ -227,6 +264,12 @@ class Broker(_EnchantObject):
         Broker and any associated Dict objects must no longer be used.
         """
         if self._this is not None:
+            # During shutdown, this finalizer may be called before
+            # some Dict finalizers.  Ensure all pointers are freed.
+            for (dict,count) in list(self._live_dicts.items()):
+                while count:
+                    self._free_dict_data(dict)
+                    count -= 1
             _e.broker_free(self._this)
             self._this = None
             
@@ -262,6 +305,10 @@ class Broker(_EnchantObject):
         if new_dict is None:
             eStr = "Dictionary for language '%s' could not be found"
             self._raise_error(eStr % (tag,),DictNotFoundError)
+        if new_dict not in self._live_dicts:
+            self._live_dicts[new_dict] = 1
+        else:
+            self._live_dicts[new_dict] += 1
         return new_dict
 
     def request_pwl_dict(self,pwl):
@@ -278,6 +325,10 @@ class Broker(_EnchantObject):
         if new_dict is None:
             eStr = "Personal Word List file '%s' could not be loaded"
             self._raise_error(eStr % (pwl,))
+        if new_dict not in self._live_dicts:
+            self._live_dicts[new_dict] = 1
+        else:
+            self._live_dicts[new_dict] += 1
         d = Dict(False)
         d._switch_this(new_dict,self)
         return d
@@ -289,11 +340,18 @@ class Broker(_EnchantObject):
         It is equivalent to calling the object's 'free' method.  Once this
         method has been called on a dictionary, it must not be used again.
         """
-        self._check_this()
-        _e.broker_free_dict(self._this,dict._this)
+        self._free_dict_data(dict._this)
         dict._this = None
         dict._broker = None
 
+    def _free_dict_data(self,dict):
+        """Free the underlying pointer for a dict."""
+        self._check_this()
+        _e.broker_free_dict(self._this,dict)
+        self._live_dicts[dict] -= 1
+        if self._live_dicts[dict] == 0:
+            del self._live_dicts[dict]
+
     def dict_exists(self,tag):
         """Check availability of a dictionary.
         
@@ -479,27 +537,30 @@ class Dict(_EnchantObject):
         may be specified using <broker>.  If not given, the default broker
         is used.
         """
-        # Superclass initialisation
-        _EnchantObject.__init__(self)
-        # Initialise object attributes to None
-        self._broker = None
-        self.tag = None
+        # Initialise misc object attributes to None
         self.provider = None
-        # Create dead object if False was given
-        if tag is False:
-            self._this = None
-        else:
+        # If no tag was given, use the default language
+        if tag is None:
+            tag = get_default_language()
             if tag is None:
-                tag = get_default_language()
-                if tag is None:
-                    err = "No tag specified and default language could not "
-                    err = err + "be determined."
-                    raise Error(err)
-            # Use module-level broker if none given
-            if broker is None:
-                broker = _broker
-            # Use the broker to get C-library pointer data
-            self._switch_this(broker._request_dict_data(tag),broker)
+                err = "No tag specified and default language could not "
+                err = err + "be determined."
+                raise Error(err)
+        self.tag = tag
+        # If no broker was given, use the default broker
+        if broker is None:
+            broker = _broker
+        self._broker = broker
+        # Now let the superclass initialise the C-library object
+        _EnchantObject.__init__(self)
+
+    def _init_this(self):
+        # Create dead object if False was given as the tag.
+        # Otherwise, use the broker to get C-library pointer data.
+        self._this = None
+        if self.tag:
+            this = self._broker._request_dict_data(self.tag)
+            self._switch_this(this,self._broker)
 
     def __del__(self):
         """Dict object destructor."""
@@ -508,7 +569,7 @@ class Dict(_EnchantObject):
             self._free()
         except AttributeError:
             pass
-            
+
     def _switch_this(self,this,broker):
         """Switch the underlying C-library pointer for this object.
         
@@ -558,8 +619,11 @@ class Dict(_EnchantObject):
         Once it has been called, the Dict object must no longer be used.
         It is called automatically when the object is garbage collected.
         """
-        if self._broker is not None:
-            self._broker._free_dict(self)
+        if self._this is not None:
+            # The broker may have been freed before the dict.
+            # It will have freed the underlying pointers already.
+            if self._broker is not None and self._broker._this is not None:
+                self._broker._free_dict(self)
 
     def check(self,word):
         """Check spelling of a word.
@@ -601,7 +665,8 @@ class Dict(_EnchantObject):
 
     def add_to_pwl(self,word):
         """Add a word to the user's personal word list."""
-        warnings.warn("Dict.add_to_pwl is deprecated, please use Dict.add",category=DeprecationWarning)
+        warnings.warn("Dict.add_to_pwl is deprecated, please use Dict.add",
+                      category=DeprecationWarning,stacklevel=2)
         self._check_this()
         word = EnchantStr(word)
         _e.dict_add_to_pwl(self._this,word.encode())
@@ -632,7 +697,9 @@ class Dict(_EnchantObject):
 
     def is_in_session(self,word):
         """Check whether a word is in the session list."""
-        warnings.warn("Dict.is_in_session is deprecated, please use Dict.is_added",category=DeprecationWarning)
+        warnings.warn("Dict.is_in_session is deprecated, "\
+                      "please use Dict.is_added",
+                      category=DeprecationWarning,stacklevel=2)
         self._check_this()
         word = EnchantStr(word)
         return _e.dict_is_in_session(self._this,word.encode())
@@ -645,6 +712,10 @@ class Dict(_EnchantObject):
         a suggestion will typically mean that <cor> appears early in the
         list of suggested spellings offered for later instances of <mis>.
         """
+        if not mis:
+            raise ValueError("can't store replacement for an empty string")
+        if not cor:
+            raise ValueError("can't store empty string as a replacement")
         self._check_this()
         mis = EnchantStr(mis)
         cor = EnchantStr(cor)
