@@ -63,12 +63,19 @@ e = None
 
 def _e_path_possibilities():
     """Generator yielding possible locations of the enchant library."""
+    # Allow it to be overridden using an environment variable.
     yield os.environ.get("PYENCHANT_LIBRARY_PATH")
+    # For linuxish systems, allow default soname lookup a chance to succeed.
+    if sys.platform not in ("win32", "darwin"):
+        yield "libenchant.so.1.6.0"
+        yield "libenchant.so.1"
+        yield "libenchant.so"
+    # See if ctypes can find the library for us, under various names.
     yield find_library("enchant")
     yield find_library("libenchant")
     yield find_library("libenchant-1")
+    # Special-case handling for enchant installed by macports.
     if sys.platform == 'darwin':
-         # enchant lib installed by macports
          yield "/opt/local/lib/libenchant.dylib"
 
 
@@ -130,13 +137,24 @@ if e is None:
 
 # No usable enchant install was found :-(
 if e is None:
-   raise ImportError("enchant C library not found")
+   msg = "The 'enchant' C library was not found. "\
+         "Please install it via your OS package manager, "\
+         "or use a pre-built binary wheel from PyPI."
+   raise ImportError(msg)
 
 
 # Define various callback function types
 
-t_broker_desc_func = CFUNCTYPE(None,c_char_p,c_char_p,c_char_p,c_void_p)
-t_dict_desc_func = CFUNCTYPE(None,c_char_p,c_char_p,c_char_p,c_char_p,c_void_p)
+def CALLBACK(restype,*argtypes):
+    """Factory for generating callback function prototypes.
+
+    This is factored into a factory so I can easily change the definition
+    for experimentation or debugging.
+    """
+    return CFUNCTYPE(restype,*argtypes)
+
+t_broker_desc_func = CALLBACK(None,c_char_p,c_char_p,c_char_p,c_void_p)
+t_dict_desc_func = CALLBACK(None,c_char_p,c_char_p,c_char_p,c_char_p,c_void_p)
 
 
 # Simple typedefs for readability
@@ -313,7 +331,7 @@ dict_describe1 = e.enchant_dict_describe
 dict_describe1.argtypes = [t_dict,t_dict_desc_func,c_void_p]
 dict_describe1.restype = None
 def dict_describe(dict,cbfunc):
-    def cbfunc1(*args):
-        cbfunc(*args[:-1])
+    def cbfunc1(tag,name,desc,file,data):
+        cbfunc(tag,name,desc,file)
     dict_describe1(dict,t_dict_desc_func(cbfunc1),None)
 
