@@ -67,13 +67,14 @@ class CmdLineChecker:
         self._stop = False
         for err in self._checker:
             self.error = err
-            printf(["ERROR:", err.word])
+            printf(["ERROR:", err.word.encode('utf8')])
             printf(["HOW ABOUT:", err.suggest()])
             status = self.read_command()
             while not status and not self._stop:
                 status = self.read_command()
             if self._stop:
-                    break
+                break
+        printf(["DONE"])
     
     def print_help(self):
         printf(["0..N:    replace with the numbered suggestion"])
@@ -88,7 +89,10 @@ class CmdLineChecker:
         printf(["HOW ABOUT:", self.error.suggest()])
     
     def read_command(self):
-        cmd = raw_input(">> ")
+        try:
+            cmd = raw_input(">> ") # Python 2.x
+        except NameError:
+            cmd = input(">> ") # Python 3.x
         cmd = cmd.strip()
         
         if cmd.isdigit():
