@@ -29,17 +29,18 @@
 #
 """
 
-    enchant.utils:    Misc utilities for the enchant package
+enchant.utils:    Misc utilities for the enchant package
+========================================================
     
-    This module provides miscellaneous utilities for use with the
-    enchant spellchecking package.  Currently available functionality
-    includes:
+This module provides miscellaneous utilities for use with the
+enchant spellchecking package.  Currently available functionality
+includes:
         
-        * string/unicode compatibility wrappers
-        * functions for dealing with locale/language settings
-        * ability to list supporting data files (win32 only)
-        * functions for bundling supporting data files from a build
-          
+    * string/unicode compatibility wrappers
+    * functions for dealing with locale/language settings
+    * ability to list supporting data files (win32 only)
+    * functions for bundling supporting data files from a build
+      
 """
 
 import os
@@ -198,6 +199,55 @@ except NameError:
     xrange = range
 
 
+#
+#  Other useful functions.
+#
+
+
+def levenshtein(s1, s2):
+    """Calculate the Levenshtein distance between two strings.
+
+    This is straight from Wikipedia.
+    """
+    if len(s1) < len(s2):
+        return levenshtein(s2, s1)
+    if not s1:
+        return len(s2)
+ 
+    previous_row = xrange(len(s2) + 1)
+    for i, c1 in enumerate(s1):
+        current_row = [i + 1]
+        for j, c2 in enumerate(s2):
+            insertions = previous_row[j + 1] + 1
+            deletions = current_row[j] + 1
+            substitutions = previous_row[j] + (c1 != c2)
+            current_row.append(min(insertions, deletions, substitutions))
+        previous_row = current_row
+ 
+    return previous_row[-1]
+
+
+
+def trim_suggestions(word,suggs,maxlen,calcdist=None):
+    """Trim a list of suggestions to a maximum length.
+
+    If the list of suggested words is too long, you can use this function
+    to trim it down to a maximum length.  It tries to keep the "best"
+    suggestions based on similarity to the original word.
+
+    If the optional "calcdist" argument is provided, it must be a callable
+    taking two words and returning the distance between them.  It will be
+    used to determine which words to retain in the list.  The default is
+    a simple Levenshtein distance.
+    """
+    if calcdist is None:
+        calcdist = levenshtein
+    decorated = [(calcdist(word,s),s) for s in suggs]
+    decorated.sort()
+    return [s for (l,s) in decorated[:maxlen]]
+    
+
+
 def get_default_language(default=None):
     """Determine the user's default language, if possible.
     
@@ -241,7 +291,9 @@ def get_resource_filename(resname):
     if os.path.exists(path):
         return path
     if hasattr(sys, "frozen"):
-        exe_path = unicode(sys.executable,sys.getfilesystemencoding())
+        exe_path = sys.executable
+        if not isinstance(exe_path, unicode):
+            exe_path = unicode(exe_path,sys.getfilesystemencoding())
         exe_dir = os.path.dirname(exe_path)
         path = os.path.join(exe_dir, resname)
         if os.path.exists(path):
